# Copyright 2012 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
#
# Copyright 2012 OpenStack Foundation
# Copyright 2012 Nebula, Inc.
# Copyright (c) 2012 X.commerce, a business unit of eBay Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from __future__ import absolute_import
import logging

from django.conf import settings
from manilaclient.v1 import client as manila_client

from horizon import exceptions
from horizon.utils.memoized import memoized  # noqa
from openstack_dashboard.api import base

LOG = logging.getLogger(__name__)


# API static values
SHARE_STATE_AVAILABLE = "available"
DEFAULT_QUOTA_NAME = 'default'


def manilaclient(request):
    insecure = getattr(settings, 'OPENSTACK_SSL_NO_VERIFY', False)
    cacert = getattr(settings, 'OPENSTACK_SSL_CACERT', None)
    try:
        manila_url = base.url_for(request, 'share')
    except exceptions.ServiceCatalogException:
        LOG.debug('no share service configured.')
        return None
    c = manila_client.Client(request.user.username,
                             input_auth_token=request.user.token.id,
                             project_id=request.user.tenant_id,
                             service_catalog_url=manila_url,
                             insecure=insecure,
                             cacert=cacert,
                             http_log_debug=settings.DEBUG)
    c.client.auth_token = request.user.token.id
    c.client.management_url = manila_url
    return c


def share_list(request, search_opts=None):
    return manilaclient(request).shares.list(search_opts=search_opts)


def share_get(request, share_id):
    return manilaclient(request).shares.get(share_id)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from openstack_dashboard.test.integration_tests import helpers
from openstack_dashboard.test.integration_tests.regions import messages


IMAGE_NAME = helpers.gen_random_resource_name("image")


class TestSaharaImageRegistry(helpers.TestCase):

    def setUp(self):
        super(TestSaharaImageRegistry, self).setUp()
        image_pg = self.home_pg.go_to_compute_imagespage()
        image_pg.create_image(IMAGE_NAME)
        image_pg.find_message_and_dismiss(messages.SUCCESS)
        image_pg.wait_until_image_active(IMAGE_NAME)

    def test_image_register_unregister(self):
        """Test the image registration in Sahara."""
        image_reg_pg = \
            self.home_pg.go_to_dataprocessing_clusters_imageregistrypage()
        image_reg_pg.register_image(IMAGE_NAME, self.CONFIG.scenario.ssh_user,
                                    "Test description")
        image_reg_pg.wait_until_image_registered(IMAGE_NAME)
        self.assertTrue(image_reg_pg.is_image_registered(IMAGE_NAME),
                        "Image was not registered.")
        self.assertTrue(
            image_reg_pg.find_message_and_dismiss(messages.SUCCESS))
        self.assertFalse(
            image_reg_pg.find_message_and_dismiss(messages.ERROR),
            "Error message occurred during image creation.")

        image_reg_pg.unregister_image(IMAGE_NAME)
        self.assertTrue(
            image_reg_pg.find_message_and_dismiss(messages.SUCCESS))
        self.assertFalse(
            image_reg_pg.find_message_and_dismiss(messages.ERROR))
        self.assertFalse(image_reg_pg.is_image_registered(IMAGE_NAME),
                         "Image was not unregistered.")

    def tearDown(self):
        image_pg = self.home_pg.go_to_compute_imagespage()
        image_pg.delete_image(IMAGE_NAME)
        super(TestSaharaImageRegistry, self).tearDown()



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging

from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import tabs

from sahara_dashboard.api import sahara as saharaclient
from sahara_dashboard.content.data_processing.clusters.image_registry \
    import tables as image_registry_tables

LOG = logging.getLogger(__name__)


class ImageRegistryTab(tabs.TableTab):
    table_classes = (image_registry_tables.ImageRegistryTable, )
    name = _("Image Registry")
    slug = "image_registry_tab"
    template_name = "horizon/common/_detail_table.html"

    def get_image_registry_data(self):
        try:
            images = saharaclient.image_list(self.request)
        except Exception:
            images = []
            msg = _('Unable to retrieve image list')
            exceptions.handle(self.request, msg)
        return images



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging

from django.core.exceptions import ValidationError
from django.utils import safestring
from django.utils.translation import ugettext_lazy as _
import six

from horizon import exceptions
from horizon import forms
from horizon import workflows
from openstack_dashboard.api import network

from sahara_dashboard.api import sahara as saharaclient


LOG = logging.getLogger(__name__)


class Parameter(object):
    def __init__(self, config):
        self.name = config['name']
        self.description = config.get('description', "No description")
        self.required = not config['is_optional']
        self.default_value = config.get('default_value', None)
        self.initial_value = self.default_value
        self.param_type = config['config_type']
        self.priority = int(config.get('priority', 2))
        self.choices = config.get('config_values', None)


def build_control(parameter):
    attrs = {"priority": parameter.priority,
             "placeholder": parameter.default_value}
    if parameter.param_type == "string":
        return forms.CharField(
            widget=forms.TextInput(attrs=attrs),
            label=parameter.name,
            required=(parameter.required and
                      parameter.default_value is None),
            help_text=parameter.description,
            initial=parameter.initial_value)

    if parameter.param_type == "int":
        return forms.IntegerField(
            widget=forms.TextInput(attrs=attrs),
            label=parameter.name,
            required=parameter.required,
            help_text=parameter.description,
            initial=parameter.initial_value)

    elif parameter.param_type == "bool":
        return forms.BooleanField(
            widget=forms.CheckboxInput(attrs=attrs),
            label=parameter.name,
            required=False,
            initial=parameter.initial_value,
            help_text=parameter.description)

    elif parameter.param_type == "dropdown":
        return forms.ChoiceField(
            widget=forms.Select(attrs=attrs),
            label=parameter.name,
            required=parameter.required,
            choices=parameter.choices,
            help_text=parameter.description)


def _create_step_action(name, title, parameters, advanced_fields=None,
                        service=None):
    class_fields = {}
    contributes_field = ()
    for param in parameters:
        field_name = "CONF:" + service + ":" + param.name
        contributes_field += (field_name,)
        class_fields[field_name] = build_control(param)

    if advanced_fields is not None:
        for ad_field_name, ad_field_value in advanced_fields:
            class_fields[ad_field_name] = ad_field_value

    action_meta = type('Meta', (object, ),
                       dict(help_text_template=("nodegroup_templates/"
                                                "_fields_help.html")))

    class_fields['Meta'] = action_meta
    action = type(str(title),
                  (workflows.Action,),
                  class_fields)

    step_meta = type('Meta', (object,), dict(name=title))
    step = type(str(name),
                (workflows.Step, ),
                dict(name=name,
                     process_name=name,
                     action_class=action,
                     contributes=contributes_field,
                     Meta=step_meta))

    return step


def build_node_group_fields(action, name, template, count, serialized=None):
    action.fields[name] = forms.CharField(
        label=_("Name"),
        widget=forms.TextInput())

    action.fields[template] = forms.CharField(
        label=_("Node group cluster"),
        widget=forms.HiddenInput())

    action.fields[count] = forms.IntegerField(
        label=_("Count"),
        min_value=0,
        widget=forms.HiddenInput())
    action.fields[serialized] = forms.CharField(
        widget=forms.HiddenInput())


def build_interface_argument_fields(
        action, name, description, mapping_type, location, value_type,
        required, default_value):
    action.fields[name] = forms.CharField(
        label=_("Name"),
        widget=forms.TextInput(),
        required=True)
    action.fields[description] = forms.CharField(
        label=_("Description"),
        widget=forms.TextInput(),
        required=False)
    action.fields[mapping_type] = forms.ChoiceField(
        label=_("Mapping Type"),
        widget=forms.Select(),
        required=True,
        choices=[("args", _("Positional Argument")),
                 ("configs", _("Configuration Value")),
                 ("params", _("Named Parameter"))])
    action.fields[location] = forms.CharField(
        label=_("Location"),
        widget=forms.TextInput(),
        required=True)
    action.fields[value_type] = forms.ChoiceField(
        label=_("Value Type"),
        widget=forms.Select(),
        required=True,
        choices=[("string", _("String")),
                 ("number", _("Number")),
                 ("data_source", _("Data Source"))])
    action.fields[required] = forms.BooleanField(
        widget=forms.CheckboxInput(),
        label=_("Required"),
        required=False,
        initial=True)
    action.fields[default_value] = forms.CharField(
        label=_("Default Value"),
        widget=forms.TextInput(),
        required=False)


def parse_configs_from_context(context, defaults):
    configs_dict = dict()
    for key, val in context.items():
        if str(key).startswith("CONF"):
            key_split = str(key).split(":")
            service = key_split[1]
            config = key_split[2]
            if service not in configs_dict:
                configs_dict[service] = dict()
            if val is None:
                continue
            if six.text_type(defaults[service][config]) == six.text_type(val):
                continue
            configs_dict[service][config] = val
    return configs_dict


def get_security_groups(request, security_group_ids):
    security_groups = []
    for group in security_group_ids or []:
        try:
            security_groups.append(network.security_group_get(
                request, group))
        except Exception:
            LOG.info(_('Unable to retrieve security group %(group)s.') %
                     {'group': group})
            security_groups.append({'name': group})

    return security_groups


def get_plugin_and_hadoop_version(request):
    plugin_name = None
    hadoop_version = None
    # In some cases request contains valuable info in both GET and POST methods
    req = request.GET.copy()
    req.update(request.POST)
    if req.get("plugin_name"):
        plugin_name = req["plugin_name"]
        hadoop_version = req["hadoop_version"]
    return plugin_name, hadoop_version


def clean_node_group(node_group):
    node_group_copy = dict((key, value)
                           for key, value in node_group.items() if value)

    for key in ["id", "created_at", "updated_at"]:
        if key in node_group_copy:
            node_group_copy.pop(key)

    return node_group_copy


def populate_image_choices(self, request, context, empty_choice=False):
    try:
        all_images = saharaclient.image_list(request)

        plugin, hadoop_version = get_plugin_and_hadoop_version(request)

        details = saharaclient.plugin_get_version_details(request,
                                                          plugin,
                                                          hadoop_version)
        choices = [(image.id, image.name) for image in all_images
                   if (set(details.required_image_tags).
                       issubset(set(image.tags)))]
    except Exception:
        exceptions.handle(request,
                          _("Unable to fetch image choices."))
        choices = []
    if empty_choice:
        choices = [(None, _('No image specified'))] + choices
    if not choices:
        choices.append(("", _("No Images Available")))

    return choices


class PluginAndVersionMixin(object):
    def _generate_plugin_version_fields(self, sahara):
        plugins = sahara.plugins.list()
        plugins = filter(is_plugin_not_hidden_for_user, plugins)
        plugin_choices = [(plugin.name, plugin.title) for plugin in plugins]

        self.fields["plugin_name"] = forms.ChoiceField(
            label=_("Plugin Name"),
            choices=plugin_choices,
            widget=forms.Select(
                attrs={"class": "plugin_name_choice switchable",
                       'data-slug': 'pluginname'}))

        for plugin in plugins:
            field_name = plugin.name + "_version"
            choice_field = forms.ChoiceField(
                label=_("Version"),
                choices=[(version, version)
                         for version in get_enabled_versions(plugin)],
                widget=forms.Select(
                    attrs={"class": "plugin_version_choice switched "
                           + field_name + "_choice",
                           "data-switch-on": "pluginname",
                           "data-pluginname-%s" % plugin.name: _("Version")})
            )
            self.fields[field_name] = choice_field


class PatchedDynamicWorkflow(workflows.Workflow):
    """Overrides Workflow to fix its issues."""

    def _ensure_dynamic_exist(self):
        if not hasattr(self, 'dynamic_steps'):
            self.dynamic_steps = list()

    def _register_step(self, step):
        # Use that method instead of 'register' to register step.
        # Note that a step could be registered in descendant class constructor
        # only before this class constructor is invoked.
        self._ensure_dynamic_exist()
        self.dynamic_steps.append(step)

    def _order_steps(self):
        # overrides method of Workflow
        # crutch to fix https://bugs.launchpad.net/horizon/+bug/1196717
        # and another not filed issue that dynamic creation of tabs is
        # not thread safe
        self._ensure_dynamic_exist()

        self._registry = dict([(step, step(self))
                               for step in self.dynamic_steps])

        return list(self.default_steps) + self.dynamic_steps


class ServiceParametersWorkflow(PatchedDynamicWorkflow):
    """Base class for Workflows having services tabs with parameters."""

    def _populate_tabs(self, general_parameters, service_parameters):
        # Populates tabs for 'general' and service parameters
        # Also populates defaults and initial values
        self.defaults = dict()

        self._init_step('general', 'General Parameters', general_parameters)

        for service, parameters in service_parameters.items():
            self._init_step(service, service + ' Parameters', parameters)

    def _init_step(self, service, title, parameters):
        if not parameters:
            return

        self._populate_initial_values(service, parameters)

        step = _create_step_action(service, title=title, parameters=parameters,
                                   service=service)

        self.defaults[service] = dict()
        for param in parameters:
            self.defaults[service][param.name] = param.default_value

        self._register_step(step)

    def _set_configs_to_copy(self, configs):
        self.configs_to_copy = configs

    def _populate_initial_values(self, service, parameters):
        if not hasattr(self, 'configs_to_copy'):
            return

        configs = self.configs_to_copy

        for param in parameters:
            if (service in configs and
                    param.name in configs[service]):
                param.initial_value = configs[service][param.name]


class StatusFormatMixin(workflows.Workflow):
    def __init__(self, request, context_seed, entry_point, *args, **kwargs):
        super(StatusFormatMixin, self).__init__(request,
                                                context_seed,
                                                entry_point,
                                                *args,
                                                **kwargs)

    def format_status_message(self, message):
        error_description = getattr(self, 'error_description', None)

        if error_description:
            return error_description
        else:
            return message % self.context[self.name_property]


class ShareWidget(forms.MultiWidget):
    def __init__(self, choices=()):
        widgets = []
        for choice in choices:
            widgets.append(forms.CheckboxInput(
                attrs={
                    "label": choice[1],
                    "value": choice[0],
                }))
            widgets.append(forms.TextInput())
            widgets.append(forms.Select(
                choices=(("rw", _("Read/Write")), ("ro", _("Read only")))))
        super(ShareWidget, self).__init__(widgets)

    def decompress(self, value):
        if value:
            values = []
            for share in value:
                values.append(value[share]["id"])
                values.append(value[share]["path"])
                values.append(value[share]["access_level"])
            return values
        return [None] * len(self.widgets)

    def format_output(self, rendered_widgets):
        output = []
        output.append("<table>")
        output.append("<tr><th>Share</th><th>Enabled</th>"
                      "<th>Path</th><th>Permissions</th></tr>")
        for i, widget in enumerate(rendered_widgets):
            item_widget_index = i % 3
            if item_widget_index == 0:
                output.append("<tr>")
                output.append(
                    "<td class='col-sm-2 small-padding'>%s</td>" %
                    self.widgets[i].attrs["label"])
            # The last 2 form field td need get a larger size
            if item_widget_index in [1, 2]:
                size = 4
            else:
                size = 2
            output.append("<td class='col-sm-%s small-padding'>" % size
                          + widget + "</td>")
            if item_widget_index == 2:
                output.append("</tr>")
        output.append("</table>")
        return safestring.mark_safe('\n'.join(output))


class MultipleShareChoiceField(forms.MultipleChoiceField):
    def validate(self, value):
        if self.required and not value:
            raise ValidationError(
                self.error_messages['required'], code='required')
        if not isinstance(value, list):
            raise ValidationError(
                _("The value of shares must be a list of values")
            )


def is_plugin_not_hidden_for_user(plugin):
    hidden_lbl = plugin.plugin_labels.get('hidden')
    if hidden_lbl and hidden_lbl['status']:
        return False
    if get_enabled_versions(plugin):
        return True
    return False


def get_enabled_versions(plugin):
    lbs = plugin.version_labels

    versions = []
    for version, data in six.iteritems(lbs):
        if data.get('enabled', {'status': True}).get('status', True):
            versions.append(version)

    if not plugin.plugin_labels.get(
            'enabled', {'status': True}).get('status', True):
        versions = []
    return versions


def is_version_of_plugin_deprecated(plugin, version):
    lbs = plugin.version_labels
    for iter_version, data in six.iteritems(lbs):
        if iter_version == version:
            if data.get('deprecated', {'status': False}).get('status', False):
                return True
            else:
                return False



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from openstack_dashboard.test.integration_tests.regions import forms
from openstack_dashboard.test.integration_tests.regions import tables
from selenium.common.exceptions import NoSuchElementException

from sahara_dashboard.test.integration_tests.pages import basepage
from sahara_dashboard.test.integration_tests.pages import mixins


class ScaleForm(forms.BaseFormRegion):
    def scale(self, node_name, new_count):
        table = self.src_elem.find_element_by_id('groups_table')
        count_input_xpath = ('.//input[@value="{}"]/ancestor::tr//'
                             'input[starts-with(@name, "count_")]'.format(
                                 node_name))
        count_input = table.find_element_by_xpath(count_input_xpath)
        count_input.clear()
        count_input.send_keys(str(new_count))


class ScaleMixin(object):
    @tables.bind_row_action('scale')
    def get_scale_form(self, button, row):
        button.click()
        return ScaleForm(self.driver, self.conf)


class ClustersPage(mixins.PluginSelectMixin, mixins.DeleteMixin,
                   basepage.BaseDataProcessingPage):

    TABLE_NAME = "clusters"
    CREATE_FIELD_MAPPING = ('cluster_name', 'description', 'cluster_template',
                            'cluster_count', 'image', 'keypair')

    TABLE_NAME_COLUMN = 'name'

    @classmethod
    def get_table_mixins(cls):
        return super(ClustersPage, cls).get_table_mixins() + (ScaleMixin,)

    @property
    def create_form(self):
        return forms.FormRegion(self.driver, self.conf,
                                field_mappings=self.CREATE_FIELD_MAPPING)

    def is_cluster_active(self, name):
        row = self._get_row_with_name(name)
        if not row:
            return False
        status_cell = row.cells['status']
        if status_cell.text.startswith('Error'):
            msg = status_cell.text
            try:
                msg = '{}\n{}'.format(
                    msg,
                    status_cell.find_element_by_tag_name('span').get_attribute(
                        'title')
                )
            except NoSuchElementException:
                pass
            raise Exception(msg)
        return status_cell.text == "Active"

    def get_cluster_instances_count(self, name):
        row = self._get_row_with_name(name)
        return int(row.cells['instances_count'].text)

    def create(self, plugin_name, plugin_version, name, description=None,
               cluster_template=None, cluster_count=1, image=None,
               keypair=None):
        self.choose_plugin(plugin_name, plugin_version)
        form = self.create_form
        form.cluster_name.text = name
        if description is not None:
            form.description.text = description
        if cluster_template is not None:
            form.cluster_template.text = cluster_template
        form.cluster_count.text = cluster_count
        if image is not None:
            form.image.text = image
        if keypair is not None:
            form.keypair.text = keypair
        form.submit()

    def scale(self, cluster_name, node_name, new_count):
        row = self._get_row_with_name(cluster_name)
        form = self.table.get_scale_form(row)
        form.scale(node_name, new_count)
        form.submit()

    def wait_until_cluster_active(self, name, timeout=None):
        self._wait_until(lambda x: self.is_cluster_active(name),
                         timeout=timeout)

    def wait_until_cluster_deleted(self, name):
        self._wait_until(lambda x: not self.is_present(name))



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json

from django.core.urlresolvers import reverse
from django.http import HttpResponse
from django.utils.translation import ugettext as _
from django.views.generic import base as django_base
from oslo_utils import timeutils
from saharaclient.api.base import APIException

from horizon import exceptions
from horizon import tabs
from horizon.utils import memoized
from horizon import workflows

from sahara_dashboard.api import sahara as saharaclient
import sahara_dashboard.content.data_processing.clusters.clusters. \
    tables as c_tables
import sahara_dashboard.content.data_processing.clusters.clusters. \
    tabs as _tabs
import sahara_dashboard.content.data_processing.clusters.clusters. \
    workflows.create as create_flow
import sahara_dashboard.content.data_processing.clusters.clusters. \
    workflows.scale as scale_flow
import sahara_dashboard.content.data_processing.clusters.clusters. \
    workflows.update as update_flow
import sahara_dashboard.content.data_processing.utils.helpers as helpers


class ClusterDetailsView(tabs.TabView):
    tab_group_class = _tabs.ClusterDetailsTabs
    template_name = 'horizon/common/_detail.html'
    page_title = "{{ cluster.name|default:cluster.id }}"

    @memoized.memoized_method
    def get_object(self):
        cl_id = self.kwargs["cluster_id"]
        try:
            return saharaclient.cluster_get(self.request, cl_id)
        except Exception:
            msg = _('Unable to retrieve details for cluster "%s".') % cl_id
            redirect = self.get_redirect_url()
            exceptions.handle(self.request, msg, redirect=redirect)

    def get_context_data(self, **kwargs):
        context = super(ClusterDetailsView, self).get_context_data(**kwargs)
        cluster = self.get_object()
        context['cluster'] = cluster
        context['url'] = self.get_redirect_url()
        context['actions'] = self._get_actions(cluster)
        return context

    def _get_actions(self, cluster):
        table = c_tables.ClustersTable(self.request)
        return table.render_row_actions(cluster)

    @staticmethod
    def get_redirect_url():
        return reverse("horizon:project:data_processing.clusters:index")


class ClusterEventsView(django_base.View):

    @staticmethod
    def _created_at_key(obj):
        return timeutils.parse_isotime(obj["created_at"])

    def get(self, request, *args, **kwargs):

        cluster_id = kwargs.get("cluster_id")
        time_helpers = helpers.Helpers(request)

        try:
            cluster = saharaclient.cluster_get(request, cluster_id,
                                               show_progress=True)
            node_group_mapping = {}
            for node_group in cluster.node_groups:
                node_group_mapping[node_group["id"]] = node_group["name"]

            provision_steps = cluster.provision_progress

            # Sort by create time
            provision_steps = sorted(provision_steps,
                                     key=ClusterEventsView._created_at_key,
                                     reverse=True)

            for step in provision_steps:
                # Sort events of the steps also
                step["events"] = sorted(step["events"],
                                        key=ClusterEventsView._created_at_key,
                                        reverse=True)

                successful_events_count = 0

                for event in step["events"]:
                    if event["node_group_id"]:
                        event["node_group_name"] = node_group_mapping[
                            event["node_group_id"]]

                    event_result = _("Unknown")
                    if event["successful"] is True:
                        successful_events_count += 1
                        event_result = _("Completed Successfully")
                    elif event["successful"] is False:
                        event_result = _("Failed")

                    event["result"] = event_result

                    if not event["event_info"]:
                        event["event_info"] = _("No info available")

                step["duration"] = time_helpers.get_duration(
                    step["created_at"],
                    step["updated_at"])
                step['started_at'] = time_helpers.to_time_zone(
                    step["created_at"], localize=True)
                result = _("In progress")
                step["completed"] = successful_events_count

                if step["successful"] is True:
                    step["completed"] = step["total"]
                    result = _("Completed Successfully")
                elif step["successful"] is False:
                    result = _("Failed")

                step["result"] = result

            status = cluster.status.lower()
            need_update = status not in ("active", "error")
        except APIException:
            # Cluster is not available. Returning empty event log.
            need_update = False
            provision_steps = []

        context = {"provision_steps": provision_steps,
                   "need_update": need_update}

        return HttpResponse(json.dumps(context),
                            content_type='application/json')


class ClusterHealthChecksView(django_base.View):
    _date_format = "%Y-%m-%dT%H:%M:%S"
    _status_in_progress = 'CHECKING'

    def _get_checks(self, cluster):
        try:
            return cluster.verification['checks']
        except (AttributeError, KeyError):
            return []

    def get(self, request, *args, **kwargs):

        time_helpers = helpers.Helpers(request)
        cluster_id = kwargs.get("cluster_id")
        need_update, not_done_count, checks = False, 0, []
        mapping_to_label_type = {'red': 'danger', 'yellow': 'warning',
                                 'green': 'success', 'checking': 'info'}
        try:
            cluster = saharaclient.cluster_get(request, cluster_id)
            for check in self._get_checks(cluster):
                check['label'] = mapping_to_label_type.get(
                    check['status'].lower())

                if not check['description']:
                    check['description'] = _("No description")

                if check['status'] == self._status_in_progress:
                    not_done_count += 1
                check['duration'] = time_helpers.get_duration(
                    check['created_at'], check['updated_at'])
                checks.append(check)
        except APIException:
            need_update = False
            checks = []
        if not_done_count > 0:
            need_update = True
        context = {"checks": checks,
                   "need_update": need_update}

        return HttpResponse(json.dumps(context),
                            content_type='application/json')


class CreateClusterView(workflows.WorkflowView):
    workflow_class = create_flow.CreateCluster
    success_url = \
        "horizon:project:data_processing.clusters:create-cluster"
    classes = ("ajax-modal",)
    template_name = "clusters/create.html"
    page_title = _("Launch Cluster")


class ConfigureClusterView(workflows.WorkflowView):
    workflow_class = create_flow.ConfigureCluster
    success_url = "horizon:project:data_processing.clusters:index"
    template_name = "clusters/configure.html"
    page_title = _("Configure Cluster")

    def get_initial(self):
        initial = super(ConfigureClusterView, self).get_initial()
        initial.update(self.kwargs)
        return initial


class ScaleClusterView(workflows.WorkflowView):
    workflow_class = scale_flow.ScaleCluster
    success_url = "horizon:project:data_processing.clusters:index"
    classes = ("ajax-modal",)
    template_name = "clusters/scale.html"
    page_title = _("Scale Cluster")

    def get_context_data(self, **kwargs):
        context = super(ScaleClusterView, self)\
            .get_context_data(**kwargs)

        context["cluster_id"] = kwargs["cluster_id"]
        return context

    def get_object(self, *args, **kwargs):
        if not hasattr(self, "_object"):
            template_id = self.kwargs['cluster_id']
            try:
                template = saharaclient.cluster_template_get(self.request,
                                                             template_id)
            except Exception:
                template = None
                exceptions.handle(self.request,
                                  _("Unable to fetch cluster template."))
            self._object = template
        return self._object

    def get_initial(self):
        initial = super(ScaleClusterView, self).get_initial()
        initial.update({'cluster_id': self.kwargs['cluster_id']})
        return initial


class UpdateClusterSharesView(workflows.WorkflowView):
    workflow_class = update_flow.UpdateShares
    success_url = "horizon:project:data_processing.clusters"
    classes = ("ajax-modal",)
    template_name = "clusters/update.html"
    page_title = _("Update Cluster Shares")

    def get_context_data(self, **kwargs):
        context = super(UpdateClusterSharesView, self)\
            .get_context_data(**kwargs)
        context["cluster_id"] = kwargs["cluster_id"]
        return context

    def get_object(self, *args, **kwargs):
        if not hasattr(self, "_object"):
            cluster_id = self.kwargs['cluster_id']
            try:
                cluster = saharaclient.cluster_get(self.request, cluster_id)
            except Exception:
                cluster = None
                exceptions.handle(self.request,
                                  _("Unable to fetch cluster."))
            self._object = cluster
        return self._object

    def get_initial(self):
        initial = super(UpdateClusterSharesView, self).get_initial()
        initial.update({
            'cluster_id': self.kwargs['cluster_id'],
            'cluster': self.get_object()})
        return initial
